package com.nokia.task;

import com.alibaba.fastjson2.JSON;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.SftpException;
import com.nokia.common.exception.MyRuntimeException;
import com.nokia.common.gpload.GploadUtil;
import com.nokia.common.gpload.entity.GploadResult;
import com.nokia.common.ssh.SSHUtil;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVPrinter;
import org.apache.commons.csv.CSVRecord;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Data
@Component
@Slf4j
@ConfigurationProperties("task")
public class SyncTask {

    /**
     * sftp ip
     */
    private String host;
    /**
     * sftp端口
     */
    private Integer port;
    /**
     * sftp账号
     */
    private String username;
    /**
     * sftp密码
     */
    private String password;
    /**
     * top用户数据文件夹
     */
    private String sourceDir;
    /**
     * 下载文件夹
     */
    private String downloadDir;
    /**
     * 去重文件名
     */
    private String distinctFilename;
    private SSHUtil sshUtil;
    private final JdbcTemplate jdbcTemplate;

    public SyncTask(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    /**
     * 同步top用户信息定时任务
     *
     */
    @XxlJob("syncTopUser")
    public void syncTopUser() {
        try {
            // 创建文件夹
            Files.createDirectories(Paths.get(downloadDir));
            sshUtil = new SSHUtil(host, port, username, password);
            // 获取文件列表
            List<String> list = sshUtil.ls(sourceDir);
            if (CollectionUtils.isEmpty(list)) {
                log.error("没有文件");
                XxlJobHelper.log("没有文件");
                XxlJobHelper.handleFail("没有文件");
                return;
            }

            String filename = list.get(list.size() - 1);
            log.info("同步的文件: {}", filename);
            XxlJobHelper.log("同步的文件: {}", filename);
            singleTask(filename);
        } catch (Exception e) {
            log.error("发生异常了: {}", e.getMessage(), e);
            XxlJobHelper.log("发生异常了: {}", e.getMessage(), e);
            XxlJobHelper.handleFail(e.getMessage());
        } finally {
            try {
                sshUtil.disconnect();
            } catch (IOException e) {
                log.error("发生异常了: {}", e.getMessage(), e);
                XxlJobHelper.log("发生异常了: {}", e.getMessage(), e);
                XxlJobHelper.handleFail(e.getMessage());
            }
        }
    }

    /**
     * 单一任务
     *
     * @param filename 文件名
     */
    public void singleTask(String filename) throws JSchException, IOException, SftpException {
        download(filename);
        distinct(filename);
        gpload();
    }

    /**
     * 下载文件
     *
     * @param filename 文件名
     */
    public void download(String filename) throws JSchException, IOException, SftpException {
        log.info("下载文件: {}", filename);
        XxlJobHelper.log("下载文件: {}", filename);
        String src = sourceDir + filename;
        String dst = downloadDir + filename;
        sshUtil.get(src, dst);
    }

    /**
     * 去重
     *
     * @param filename 文件名
     */
    public void distinct(String filename) throws IOException {
        log.info("文件 {} 去重", filename);
        XxlJobHelper.log("文件 {} 去重", filename);
        String inputFilePath = downloadDir + filename;
        Path inputPath = Paths.get(inputFilePath);
        List<Object> modifiedUsers = new ArrayList<>();
        try (CSVParser parser = CSVFormat.DEFAULT.builder().build()
                .parse(new InputStreamReader(Files.newInputStream(inputPath), "gbk"));
             OutputStreamWriter osw = new OutputStreamWriter(Files.newOutputStream(Paths.get(distinctFilename)),
                     StandardCharsets.UTF_8);
             CSVPrinter printer = new CSVPrinter(osw, CSVFormat.DEFAULT)) {
//            printer.printRecord("login_name","org_id","org_name","user_id","user_name","phone","employee_code","province_id","city_id","area_id", "province_name", "city_name", "area_name");
            Map<String, CSVRecord> csvRecordMap = new HashMap<>();
            // 按loginName去重
            for (CSVRecord t : parser) {
                if (t.getRecordNumber() == 1) {
                    continue;
                }
                csvRecordMap.put(t.get(4), t);
            }
            log.info("{} 条数据", csvRecordMap.size());
            XxlJobHelper.log("{} 条数据", csvRecordMap.size());
            if (CollectionUtils.isEmpty(csvRecordMap)) {
                throw new MyRuntimeException("数据为空");
            }
            // 查询所有用户，过滤已删除和测试用户
            String sql = "select user_id, login_name, city_id from sqmdb_rpt.acl_user"
                    + " where deleted = 0 and test_user = 0 order by user_id";
            List<Map<String, Object>> allUsers = jdbcTemplate.queryForList(sql);
            Map<Object, Map<String, Object>> userMap = new HashMap<>();
            allUsers.forEach(t -> userMap.put(t.get("login_name"), t));
            // 查询所有地区
            sql = "select * from sqmdb_rpt.acl_area order by type_code, area_id";
            List<Map<String, Object>> areas = jdbcTemplate.queryForList(sql);
            Map<String, Map<String, Object>> cityMap = new HashMap<>();
            Map<Object, List<Map<String, Object>>> areaMap = new HashMap<>();
            for (Map<String, Object> t : areas) {
                if (t.get("type_code").equals(2)) {
                    cityMap.put((String) t.get("area_name"), t);
                    areaMap.putIfAbsent(t.get("area_id"), new ArrayList<>());
                    for (Map<String, Object> tt : areas) {
                        if (tt.get("type_code").equals(3) && tt.get("parent_id").equals(t.get("area_id"))) {
                            areaMap.get(t.get("area_id")).add(tt);
                        }
                    }
                }
            }
            for (CSVRecord t : csvRecordMap.values()) {
                String orgId = t.get(0);
                String orgName = t.get(1);
                String userId = t.get(2);
                String userName = t.get(3);
                String loginName = t.get(4);
                String phone = t.get(5);
                String employeeCode = t.get(6);
                Integer provinceId = -1;
                Object cityId = -1;
                Object areaId = null;
//                String provinceName = "河北省";
//                Object cityName = null;
//                Object areaName = null;
                for (Map.Entry<String, Map<String, Object>> tt : cityMap.entrySet()) {
                    // 匹配地市
                    if (orgName.contains(tt.getKey())) {
                        Map<String, Object> area = tt.getValue();
                        cityId = area.get("area_id");
//                        cityName = area.get("area_name");
                        List<Map<String, Object>> areaList = areaMap.get(area.get("area_id"));
                        for (Map<String, Object> ttt : areaList) {
                            // 匹配区县
                            if (orgName.contains((CharSequence) ttt.get("area_name"))) {
                                areaId = ttt.get("area_id");
//                                areaName = ttt.get("area_name");
                                break;
                            }
                        }
                        break;
                    }
                }
//                printer.printRecord(loginName, orgId, orgName, userId, userName, phone, employeeCode, provinceId, cityId, areaId, provinceName, cityName, areaName);
                printer.printRecord(loginName, orgId, orgName, userId, userName, phone, employeeCode, provinceId,
                        cityId, areaId);
                // 检查用户信息变化
                Map<String, Object> user = userMap.get(loginName);
                if (user != null && !cityId.equals(user.get("city_id"))) {
                    modifiedUsers.add(user.get("user_id"));
                    log.debug("用户 {} 地市变化: {} -> {}", user.get("user_id"), user.get("city_id"), cityId);
                }
            }
            log.info("文件 {} 去重完成", filename);
            XxlJobHelper.log("文件 {} 去重完成", filename);
            // 删除本地源文件
            Files.deleteIfExists(inputPath);
            log.info("删除本地源文件 {}", filename);
            XxlJobHelper.log("删除本地源文件 {}", filename);
            // 检查删除用户
            sql = "select user_id from sqmdb_rpt.acl_user"
                    + " where deleted = 0 and test_user = 0 and login_name not in (:loginNames)"
                    + " order by user_id";
            Map<String, Object> paramMap = new HashMap<>();
            paramMap.put("loginNames", csvRecordMap.keySet());
            NamedParameterJdbcTemplate namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(jdbcTemplate);
            List<Integer> deletedUsers = namedParameterJdbcTemplate.queryForList(sql, paramMap, Integer.class);
            if (!CollectionUtils.isEmpty(deletedUsers)) {
                log.error("已删除用户: {}", JSON.toJSONString(deletedUsers));
            }
            // 检查权限地市
            sql = "select distinct au.user_id"
                    + " from sqmdb_rpt.acl_user au"
                    + " inner join sqmdb_rpt.acl_user_role_city aurc on au.user_id = aurc.user_id"
                    + " where au.city_id != aurc.city_id"
                    + " and aurc.role_id != 3"
                    + " order by au.user_id";
            List<Integer> diffUsers = jdbcTemplate.queryForList(sql, Integer.class);
            if (!CollectionUtils.isEmpty(diffUsers)) {
                log.error("权限地市不一致用户: {}", JSON.toJSONString(diffUsers));
            }
            if (!CollectionUtils.isEmpty(modifiedUsers)) {
                log.error("地市变化用户: {}", JSON.toJSONString(modifiedUsers));
            }
        }
    }

    public void gpload() throws IOException {
        String gploadCommand = "sh gpload/gpload.sh";
        GploadResult gpload = GploadUtil.gpload(gploadCommand);
        if (Boolean.TRUE.equals(gpload.getTaskStatus())) {
            log.info("gpload完成: {}", gpload);
            XxlJobHelper.log("gpload完成: {}", gpload);
            // 删除文件
            Files.deleteIfExists(Paths.get(distinctFilename));
        } else {
            throw new MyRuntimeException("gpload失败: " + gpload.getMessage());
        }
    }
}
